//Adaptnoise.c  Adaptive filter with C6416 DSK for noise cancellation
#include "dsk6416_aic23.h"
Uint32 fs=DSK6416_AIC23_FREQ_8KHZ; 	//set sampling rate
#include "refnoise.h" 			  		//cosine 312 Hz
#include "dplusn.h"           	  	//sin(1500) + sin(312)
#define beta 0.01        		  		//rate of convergence
#define N 30                  	  	//# of weights (coefficients)
#define NS 128                	  	//# of output sample points
int w[N];                  	      //buffer for weights of adapt filter
int delay[N];              	      //input buffer to adapt filter
int output;		                     //overall output
int yn, E;               		  		//output of adapt filter,"error" signal
short out_type = 1;         	      //output type for slider

interrupt void c_int11()      	   //ISR
 {
  short i;
  static short buffercount=0;		   //init count of # out samples
  delay[0] = refnoise[buffercount]; //cos(312Hz) input to adapt FIR
  yn = 0;                           //init output of adapt filter
  for (i = 0; i < N; i++)           //to calculate out of adapt FIR
	yn += (w[i] * delay[i]);         //output of adaptive filter
  yn = yn/1e7;								//scaled output of adaptive FIR
  E = dplusn[buffercount] - yn;     //"error" signal=(d+n)-yn
  for (i = N-1; i >= 0; i--)        //to update weights and delays
   {
	w[i] = w[i] + beta*E*delay[i];   //update weights
   	delay[i] = delay[i-1];        //update delay samples
   }
  buffercount++;                    //increment buffer count
  if (buffercount >= NS) 	         //if buffercount=# out samples
    	buffercount = 0;            	//reinit count
  if (out_type == 1)		     	  		//if slider in position 1
	 output = ((short)E);        		//"error" signal as overall output
  else if (out_type == 2)
    output = dplusn[buffercount]; 	//desired(1500)+noise(312)
  output_sample(output);	        	//overall output result
  return;
}

 void main()
{
  short T = 0;
  for (T = 0; T < 30; T++)
   {
   	w[T] = 0;				  			//init buffer for weights
   	delay[T] = 0;			  			//init buffer for delay samples
   }
  comm_intr();                 	  	//init DSK, codec, McBSP
  while(1);                    	  	//infinite loop
}
