/******************************************************************************
 FILE
 	digitgen.c - This is the C source code for a function used to generate
	index tables for a digit reversal function for a radix-4 FFT.    
	
	Recall that the DIF Radix 4 method requires the transformed values to be bit
	reversed in a "mod-4" fashion.  Thus, in a 16-bit transform, element 1001 
	must be swapped with 0110, and element 1010 would be swapped with
	1010 (and hence is not swapped at all).  Thus for an transform with 
	n = 16 points, the following values would be generated by this function:
	
	                Iindex[0] = %0001 = 1      Jindex[0] = %0100 = 4
	                Iindex[1] = %0010 = 2      Jindex[1] = %1000 = 8
	                Iindex[2] = %0011 = 3      Jindex[2] = %1100 = 12
	                Iindex[3] = %0110 = 6      Jindex[3] = %1001 = 9
	                Iindex[4] = %0111 = 7      Jindex[4] = %1101 = 13
	                Iindex[5] = %1011 = 11     Jindex[5] = %1110 = 14
	                
	                count = 5   (=> 6 pairs of elements are to be swapped!)
 ******************************************************************************/
 

void R4DigitRevIndexTableGen(int n, int * count, unsigned short *IIndex, unsigned short *JIndex)
{
 	int j, n1, k, i;
 	j = 1;
	n1 = n - 1;
	*count = 0;

	for(i=1; i<=n1; i++)
	{
		if(i < j)
		{
			IIndex[*count] = (unsigned short)(i-1);
			JIndex[*count] = (unsigned short)(j-1);
			*count = *count + 1;
		}
					  
		k = n >> 2;

		while(k*3 < j)
		{
			j = j - k*3;
			k = k >> 2;

		}
		j = j + k;
	}
}


/******************************************************************************
 FILE
 	digit.c - This is the C source code for a digit reversal function for 
	a radix-4 FFT
 ******************************************************************************/


void digit_reverse(double *yx, unsigned short *JIndex, unsigned short *IIndex, int count)
{   

   /* NOTE TRICK -- The value pointed at by yx (*yx) is 
      declared to be of size double... thus both the real and imaginary part
      of each pair of transformed value are exchanged at the same time! */

	int i;
	unsigned short I, J;
	double YXI, YXJ;

	for (i = 0; i<count; i++)
	{
		I = IIndex[i];
		J = JIndex[i];
		YXI = yx[I];
		YXJ = yx[J];
		yx[J] = YXI;
		yx[I] = YXJ;
	}

}


